/// <reference types="@fibjs/types" />
Object.defineProperty(exports, "__esModule", { value: true });
const url = require("url");
const util = require("util");
const net = require("net");
const uuid = require("uuid");
const ParseQSDotKey = require("parse-querystring-dotkey");
const FibPool = require("fib-pool");
function driverUUid() {
    return uuid.node().hex();
}
exports.driverUUid = driverUUid;
function filterDriverType(protocol) {
    switch (protocol) {
        case 'sqlite:':
            return 'sqlite';
        case 'mysql:':
            return 'mysql';
        case 'redis:':
            return 'redis';
        case 'mongodb:':
            return 'mongodb';
        default:
            return 'unknown';
    }
}
exports.filterDriverType = filterDriverType;
function forceInteger(input, fallback) {
    try {
        input = parseInt(input);
    }
    catch (error) {
        input = null;
    }
    if (input === null || isNaN(input))
        input = fallback;
    return input;
}
exports.forceInteger = forceInteger;
function castQueryStringToBoolean(input) {
    switch (input) {
        case "1":
        case "true":
        case "y":
            return true;
        case "0":
        case "false":
        case "no":
        case "n":
        case "":
            return false;
        default:
            return !!input;
    }
}
exports.castQueryStringToBoolean = castQueryStringToBoolean;
function unPrefix(str = '', prefix = '/') {
    if (!str || typeof str !== 'string')
        return '';
    if (str.slice(0, prefix.length) === prefix)
        str = str.slice(prefix.length);
    return str;
}
function ensureSuffix(str = '', suffix = '//') {
    if (!str || typeof str !== 'string')
        return '';
    const lidx = str.lastIndexOf(suffix);
    if (str.slice(lidx) !== suffix)
        str += suffix;
    return str;
}
exports.ensureSuffix = ensureSuffix;
function parseConnectionString(input) {
    input = input || {};
    let urlObj = input instanceof net.Url ? input : null;
    if (typeof input === 'string') {
        urlObj = url.parse(input);
        input = {
            protocol: urlObj.protocol || null,
            slashes: urlObj.slashes || false,
            query: urlObj.query || null,
            username: urlObj.username || null,
            password: urlObj.password || null,
            host: urlObj.host || null,
            hostname: urlObj.hostname || null,
            port: urlObj.port || null,
            href: urlObj.href || null,
            database: unPrefix(urlObj.pathname, '/') || null,
            pathname: urlObj.pathname || null,
        };
    }
    else if (typeof input !== 'object') {
        input = {};
    }
    if (input.user && !input.username)
        input.username = input.user;
    delete input.user;
    if (typeof input.query === 'string')
        input.query = ParseQSDotKey(input.query);
    input.query = Object.assign({}, input.query);
    input = Object.assign({}, input);
    input = util.pick(input, [
        'protocol',
        'slashes',
        'query',
        'database',
        'username',
        'password',
        'host',
        'hostname',
        'port',
        'href',
        'pathname',
    ]);
    input.slashes = !!input.slashes;
    input.port = forceInteger(input.port, null);
    return input;
}
exports.parseConnectionString = parseConnectionString;
function parsePoolConfig(input) {
    if (!input || input === true)
        return {};
    if (typeof input !== 'object')
        return {};
    const { maxsize = undefined, timeout = undefined, retry = undefined } = (input || {});
    return {
        maxsize,
        timeout,
        retry
    };
}
exports.parsePoolConfig = parsePoolConfig;
function mountPoolToDriver(driver, poolSetting = driver.config.pool) {
    if (!driver.pool && poolSetting)
        driver.pool = FibPool(Object.assign({ create: () => {
                return driver.getConnection();
            }, destroy: (conn) => {
                return conn.close();
            } }, parsePoolConfig(poolSetting)));
}
exports.mountPoolToDriver = mountPoolToDriver;
function arraify(item) {
    return Array.isArray(item) ? item : [item];
}
exports.arraify = arraify;
